
var ko;
var currentDocument;
var numSelectedItems;
var origGlobalTestPlanNames;
var origDocumentTestPlanNames;

var dialog = null;

var handleSelect = function(ev) {
    var enableButtons = ev.target.selectedCount > 0;
    if (enableButtons) {
        dialog.copyButton.removeAttribute("disabled");
        dialog.moveButton.removeAttribute("disabled");
    } else {
        dialog.copyButton.setAttribute("disabled", true);
        dialog.moveButton.setAttribute("disabled", true);
    }
};

function importGlobalTestPlanLoad() {
    ko = window.arguments[0];
    var docURI = window.arguments[1];
    dialog = {
        copyButton: document.getElementById('copy'),
        moveButton: document.getElementById('move'),
        fromList: document.getElementById('importGlobal_fromList'),
        toList: document.getElementById('importGlobal_toList'),
        __END__ : null
    };
        
    origGlobalTestPlanNames = ko.sleuth.manager._getGlobalTestPlanNames(true);
    if (!origGlobalTestPlanNames || origGlobalTestPlanNames.length == 0) {
        require("notify/notify").send("No global test plans found", "unittest",
                                      {priority: "warning"});
        window.close();
        return;
    }
    currentDocument = ko.views.manager.getViewForURI(docURI).koDoc;
    if (!currentDocument) {
        require("notify/notify").send("Can't find document " + docURI, "unittest",
                                      {priority: "warning"});
        window.close();
        return;
    }
    origDocumentTestPlanNames =
        ko.sleuth.manager._getProjectTestPlanNames(currentDocument, 'prefs', true);
    var i = 0;
    for (i = 0; i < origGlobalTestPlanNames.length; i++) {
        var listitem = dialog.fromList.appendItem(origGlobalTestPlanNames[i]);
        listitem.crop = 'center';
        if (origDocumentTestPlanNames.indexOf(origGlobalTestPlanNames[i]) >= 0) {
            listitem.disabled = true;
        }
    }
    for (i = 0; i < origDocumentTestPlanNames.length; i++) {
        var listitem = dialog.toList.appendItem(origDocumentTestPlanNames[i]);
        listitem.crop = 'center';
    }
    var toListLabel = document.getElementById("importGlobal_toList_label");
    var labelText = toListLabel.value;
    var fixedLabelText = labelText.replace('%(basename)',
                                           currentDocument.file.leafName);
    toListLabel.value = fixedLabelText;
    
    dialog.fromList.addEventListener('select', handleSelect, false);
}

var log = ko.logging.getLogger("sleuth.importGlobals");
log.setLevel(ko.logging.LOG_DEBUG);

function importGlobalTestPlanOK() {
    try {
        importGlobalTestPlanOK_Aux();
    } catch(ex) {
        for (var p in ex) {
            var o = ex[p];
            if (typeof(o) != "function") {
                dump("ex[" + p + "]=" + o + "\n" )
            }
        }
        log.exception(ex);
    }
}

function importGlobalTestPlanOK_Aux() {
    // Look for names that have changed.
    var globalTestPlanNamesToDelete = [];
    var documentTestPlanNamesToAdd = [];
    for (var i = 0; i < dialog.toList.getRowCount(); i++) {
        var testPlanName = dialog.toList.getItemAtIndex(i).label;
        if (origDocumentTestPlanNames.indexOf(testPlanName) == -1) {
            documentTestPlanNamesToAdd.push(testPlanName);
        }
    }
    
    // See what's been pulled out:
    var remainingGlobalTestNames = [];
    for (i = 0; i < dialog.fromList.getRowCount(); i++) {
        remainingGlobalTestNames.push(dialog.fromList.getItemAtIndex(i).label);
    }
    for (i = 0; i < origGlobalTestPlanNames.length; i++) {
        var testPlanName = origGlobalTestPlanNames[i];
        if (remainingGlobalTestNames.indexOf(testPlanName) == -1) {
            globalTestPlanNamesToDelete.push(testPlanName);
        }
    }
    for (i = 0; i < dialog.fromList.getRowCount(); i++) {
        var testPlanName = dialog.fromList.getItemAtIndex(i).label;
        if (origGlobalTestPlanNames.indexOf(testPlanName) == -1) {
            globalTestPlanNamesToDelete.push(testPlanName);
        }
    }
    if (documentTestPlanNamesToAdd.length == 0
        && globalTestPlanNamesToDelete.length == 0) {
        return;
    }
    var TESTPLAN_PREFNAME = Components.classes["@activestate.com/koSleuthService;1"].
            getService(Components.interfaces.koISleuthService).TESTPLAN_PREFNAME;
    var prefSvcClass = Components.classes["@activestate.com/koPrefService;1"];
    var preferenceSetClass = Components.classes["@activestate.com/koPreferenceSet;1"];
    var gprefs = prefSvcClass.getService(Components.interfaces.koIPrefService).prefs;
    var globalTestPlanPrefs = gprefs.getPref(TESTPLAN_PREFNAME);
    if (documentTestPlanNamesToAdd.length > 0) {
        var docPrefs = currentDocument.prefs;
        var docTestPlanPrefs;
        if (!docPrefs.hasPrefHere(TESTPLAN_PREFNAME)) {
            docTestPlanPrefs = preferenceSetClass.createInstance();
            docPrefs.setPref(TESTPLAN_PREFNAME, docTestPlanPrefs);
        } else {
            docTestPlanPrefs = docPrefs.getPref(TESTPLAN_PREFNAME);
        }
    }
    var testPlanName, globalTestPlanPref, thisTestPlanPref, tpNames = null;
    // Copy new document prefs in...
    for (i = 0; i < documentTestPlanNamesToAdd.length; i++) {
        testPlanName = documentTestPlanNamesToAdd[i];
        globalTestPlanPref = globalTestPlanPrefs.getPref(testPlanName);
        thisTestPlanPref = preferenceSetClass.createInstance();
        if (!tpNames) {
            tpNames = globalTestPlanPref.getPrefIds();
        }
        for (var j = 0; j < tpNames.length; j++) {
            var tpName = tpNames[j];
            thisTestPlanPref.setStringPref(tpName,
                                           globalTestPlanPref.getStringPref(tpName));
        }
        docTestPlanPrefs.setPref(testPlanName, thisTestPlanPref);
    }
    // Remove deleted global prefs    
    for (i = 0; i < globalTestPlanNamesToDelete.length; i++) {
        testPlanName = globalTestPlanNamesToDelete[i];
        globalTestPlanPrefs.deletePref(testPlanName);
    }
    tpNames = globalTestPlanPrefs.getPrefIds();
    if (tpNames.length == 0) {
        gprefs.deletePref(TESTPLAN_PREFNAME);
    }
}

function importGlobalTestPlanCancel() {
    // Nothing to do
}

function doCopyOrMoveSelectedItems(doMove) {
    var listitems = dialog.fromList.selectedItems;
    for (var i = 0; i < listitems.length; i++) {
        var listitem = listitems[i];
        if (!listitem.disabled) {
            dialog.toList.appendItem(listitem.label);
        }
        listitem.disabled = true;
    }
    if (doMove) {
        for (i = listitems.length - 1; i >= 0; i--) {
            var listitem = listitems[i];
            dialog.fromList.removeItemAt(dialog.fromList.getIndexOfItem(listitem));
        }
    }
    dialog.fromList.clearSelection();
}

